<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

class SmartsubModelAccounts extends JModelList
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'title', 'a.title',
				'alias', 'a.alias',
				'checked_out', 'a.checked_out',
				'checked_out_time', 'a.checked_out_time',
				'published', 'a.published',
				'access', 'a.access', 'access_level',
				'created', 'a.created',
				'ordering', 'a.ordering',
				'is_default', 'a.is_default',
				'amount', 'a.amount',
				'currencyID', 'a.currencyID',
				'valid_period', 'a.valid_period',
				'valid_period_unit', 'a.valid_period_unit',
				'group_title', 'ug.title'
			);
		}

		parent::__construct($config);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		// Load the filter state.
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$accessId = $this->getUserStateFromRequest($this->context . '.filter.access', 'filter_access', null, 'int');
		$this->setState('filter.access', $accessId);

		$published = $this->getUserStateFromRequest($this->context . '.filter.state', 'filter_state', '', 'string');
		$this->setState('filter.state', $published);

		// Load the parameters.
		$params = JComponentHelper::getParams('com_smartsub');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.ordering', 'asc');
	}

	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.access');
		$id .= ':' . $this->getState('filter.state');
		
		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 * @since   1.6
	 */
	protected function getListQuery()
	{
		// Create a new query object
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$user = JFactory::getUser();

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.id, a.title, a.alias, a.checked_out, a.checked_out_time, a.params, ' .
					'a.is_default, a.amount, a.published, a.access, a.ordering, ' .
					'a.description, a.currencyID, a.valid_period, a.valid_period_unit'
			)
		);
		$query->from($db->quoteName('#__ss_accounts') . ' AS a');
		
		// Join over the users for the checked out user
		$query->select('uc.name AS editor')
			->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over the asset groups
		$query->select('ag.title AS access_level')
			->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');

		// Join over the user groups
		$query->select('ug.title AS group_title')
			->join('LEFT', '#__usergroups AS ug ON ug.id = a.group_id');
		
		// Filter by access level.
		if ($access = $this->getState('filter.access'))
		{
			$query->where('a.access = ' . (int) $access);
		}

		// Implement View Level Access
		if (!$user->authorise('core.admin'))
		{
			$groups = implode(',', $user->getAuthorisedViewLevels());
			$query->where('a.access IN (' . $groups . ')');
		}

		// Filter by published state
		$published = $this->getState('filter.state');
		if (is_numeric($published))
		{
			$query->where('a.published = ' . (int) $published);
		}
		elseif ($published === '')
		{
			$query->where('(a.published IN (0, 1))');
		}

		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('a.id = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->quote('%' . $db->escape($search, true) . '%');
				$query->where('(a.title LIKE ' . $search . ' OR a.alias LIKE ' . $search . ')');
			}
		}

		// Add the list ordering clause.
		$orderCol = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		$query->order($db->escape($orderCol . ' ' . $orderDirn));

		$query->group('a.id');
		
		//echo nl2br(str_replace('#__','jos_',$query));
		return $query;
	}
	
	public function getItems()
	{
		$items = parent::getItems();
		if(empty($items))
		{
			return $items;
		}
		
		// Populate items with params. We assume that the number of accounts will be moderate
		// in all usage scenarios, so that this loop will not affect performance.
		// If we detect an important performance issue, we will have to refactor the DB structure
		$global_params = JComponentHelper::getParams('com_smartsub');
		foreach($items as &$item)
		{
			$params = new JRegistry();
			$params->loadString($item->params);
			$item->discount_unused_time = $params->get('discount_unused_time', -1);
			$item->discount_on_downgrade = $params->get('discount_on_downgrade', -1);
			$item->lock_expires_on_upgrade = $params->get('lock_expires_on_upgrade', -1);
			$item->discount_on_renew = $params->get('discount_on_renew', -1);
			foreach(array('discount_unused_time', 'discount_on_downgrade', 'lock_expires_on_upgrade', 'discount_on_renew') as $param_key)
			{
				if($item->$param_key == -1)
				{
					$item->$param_key = $global_params->get($param_key);
				}
			}
		}
		return $items;
	}
}
